/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_CENTIPEDE_FACTORY_H
#define HPP_LOS_CENTIPEDE_FACTORY_H

#include <cstdio>
#include <list>
#include <vector>
#include "MatrixAbstractLayer/MatrixAbstractLayer.h" 
class ChppLosCentipede;
class ChppHikLinearSystem;
class ChppLosFootprint;
class ChppGikStandingRobot;
class ChppLosFootprintShape;
class ChppLosFootprintArea;
class ChppLosTask;
class ChppLosLink;

/**
A constructor of ChppLosCentipede objects
\ingroup parts
 */
class ChppLosCentipedeFactory
{
    public:

        /**
        Springman constructor
        A springman is a ChppGikStandingRobot in double support to which a chain of ChppLoSupportPolygonLink objects is attached. The chain acts as the walking steps from a start support polygon to the standingrobot's current support polygon.
        \param initialleftprint initial support polygon's left foot
        \param initialrightprint initial support polygon's right foot
        \param inStandingRobot gik standing robot
        \param numberExtraVirtualStepLinks The number of extra virtual step links connected to the humanoid model
        \param rootIsRight the root foot for the first support polygon
        \param leftStepArea area authorized for left step with respect to right foot
        \param rightStepArea area authorized for right step with respect to left foot
        \param inLeftFootShape shape of left footprint
        \param inRightFootShape shape of right footprint
        \param outCentipedeParts output vector that will hold pointers to the parts created by this constructor and assembled in the centipede
        \param outCentipedeConstraints output vector that will hold pointers to the standard constraints created by this constructor and for the centipede
         */
        static ChppLosCentipede* createSpringman ( const ChppLosFootprint& initialleftprint, const ChppLosFootprint& initialrightprint, ChppGikStandingRobot& inStandingRobot, unsigned int numberExtraVirtualStepLinks, bool rootIsRight, const ChppLosFootprintArea& leftStepArea,  const ChppLosFootprintArea& righttStepArea, const ChppLosFootprintShape& inLeftFootShape, const ChppLosFootprintShape& inRightFootShape, std::list<ChppLosLink*>& outCentipedeParts, vectorN& outMask);
        
        static void extendSpringman (ChppLosCentipede& inCentipede, unsigned int numberExtraLinks, std::list<ChppLosLink*>& io_centipedeParts, std::list<ChppLosTask*>& io_outCentipedeConstraints, vectorN& outMask);
        
        static void createCentipedeConstraints(ChppLosCentipede* inCentipede, std::list<ChppLosTask*>& outCentipedeConstraints);


    private:
        static void addLink ( ChppLosLink* inLink, unsigned int& ndof, vectorN& config, vectorN& mask,  std::list<ChppLosLink*>& centipedeParts, ChppLosCentipede* centipede, ChppGikStandingRobot* inRobot ); 
};

#endif
