/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_LINKGEOMETRY_H
#define HPP_LOS_LINKGEOMETRY_H

#include <vector>
#include "hpplocalstepper/hppLosGeometry.hh"
class CjrlJoint;
class ChppLosCentipede;
class ChppLosLink;
class ChppLosRobotLink;
class ChppLosFootprintShape;

class LosCentipedeLinkGeometry: virtual public LosGeometry
{
    public:
        LosCentipedeLinkGeometry ( ChppLosCentipede& inCentipede, ChppLosLink& inLink );
        ChppLosCentipede& centipede();
        ChppLosLink& link();
    protected:
        ChppLosCentipede* attCentipede;
        ChppLosLink* attLink;
};

class LosRobotLinkGeometry: public LosCentipedeLinkGeometry
{
    public:
        LosRobotLinkGeometry ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inLink, CjrlJoint& jointJ );
        ChppLosRobotLink& robotlink();
        CjrlJoint& joint();
    protected:
        ChppLosRobotLink* attRobotLink;
        CjrlJoint* attJoint;
};

class LosRobotCylinder:  public LosRobotLinkGeometry, public LosCylinder
{
    public:
        LosRobotCylinder ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inLink, CjrlJoint& jointJ, const vector3d& pointAinJ, const vector3d& pointBinJ,  double inRadius ) ;
};

class LosFootprint:  public LosCentipedeLinkGeometry
{
    public:
        LosFootprint ( ChppLosCentipede& inCentipede, ChppLosLink& inLink, const ChppLosFootprintShape& inShape );
        const ChppLosFootprintShape& localshape() const ;
        const std::vector<vector3d>& vertices() const ;
        void transform();
    private:
        const ChppLosFootprintShape* lshp;
        std::vector<vector3d> attVertices;
        ChppLosLink* attLink;
        ChppLosCentipede* attCentipede;
};

/*
class LosFixedObject : public virtual LosGeometry
{
    public:
        virtual std::vector<vector3d> shape();
    protected:
        std::vector<vector3d> attShape;
};

class LosFixedHalfspace: public virtual LosFixedObject, public LosHalfspace
{
    public:

        LosFixedHalfspace ( const vector3d& inBorderPoint, const vector3d& inNormalOut ) ;
};

class LosFixedCylinder: public LosFixedObject, public LosCylinder
{
    public:
        LosFixedCylinder ( const vector3d& pointA, const vector3d& pointB, double inRadius );
};
*/
#endif
