/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_COLLISION_AVOIDER_H
#define HPP_LOS_COLLISION_AVOIDER_H

#include "hpplocalstepper/tasks/hppLosTask.hh"
class LosRobotCylinder;
class LosFixedCylinder;
class LosFootprint;
class LosFixedHalfspace;
class ChppLosRobotLink;
class LosCylinder;
class LosHalfspace;
class LosGeometry;
class LosRobotLinkGeometry;
class LosCentipedeLinkGeometry;


/**
Stores a pair of points' positions whose relative linear velocity are constrained to avoid collision.
 */
class PointPair
{
    public:
        vector3d point1, point1abs;
        vector3d point2, point2abs;
        vector3d normalp1p2;
        double distance;
	 std::ostream & print( std::ostream & out ) const
{
out << "lp1 [" << point1[0] <<"; "<< point1[1] <<"; "<< point1[2] <<"] "<< "  lp2 [" << point2[0] <<"; "<< point2[1] <<"; "<< point2[2] <<"]   distance " << distance << std::endl;
return out;
}
};

class VelocityDamper
{
    public:
        double Di,Ds,Di_Ds,Xi;
        std::vector< PointPair > pointpairs;
};

class SelfDamper: public VelocityDamper
{
    public:
        LosRobotCylinder *object1, *object2;
};

class ObstacleDamper: public VelocityDamper
{
    public:
        LosRobotLinkGeometry *robject;
        LosCentipedeLinkGeometry *lobject;
        LosGeometry *fobject;
};


/**
Calculates inequality constraints to avoid collision between selected pairs of objects
\ingroup tasks
 */
class ChppLosCollisionAvoider: public ChppLosTask
{
    public:
        /**
        \brief Constructor
         */
        ChppLosCollisionAvoider ( ChppLosCentipede& inCentipede );

        /**
        \brief Set a pair of a robot cylinder and a fixed object to be constrained against collision
        \param inDi distance between objects below which their relative velocity is constrained
        \param inDs minimal distance between objects
        \param inXi maximal velocity at distance inDi
         */
        bool setObstaclePair ( LosCentipedeLinkGeometry& inC, LosGeometry& inFixedOb, double inDi, double inDs, double inXi );
        /**
        \brief Set a pair of robot objects to be constrained against collision
        \param inDi distance between objects below which their relative velocity is constrained
        \param inDs minimal distance between objects
        \param inXi maximal velocity at distance inDi
        \return false is one of the objects is not found
         */
        void setSelfPair ( LosRobotCylinder& inC1, LosRobotCylinder& inC2, double inDi, double inDs, double inXi );

        /**
        \brief Inherited. All constraints returned are ineqaulity constraints
         */
        bool isEqualityTask() const {return false;}
        /**
        \brief Inherited.
         */
        void computeDifferentialSystem();
        /**
        \brief Inherited.
         */
        const vectorN& currentValue();


        void getCylinderPointPairs ( const LosCylinder& inC1, const LosCylinder& inC2, std::vector< PointPair >& outPointPairs );

        void getCylinderHalfspacePointPairs ( const LosCylinder& inC, const LosHalfspace& inH, std::vector< PointPair >& outPointPairs );

        void getFootprintHalfspacePointPair ( const std::vector<vector3d>& inV, const LosHalfspace& inH, std::vector< PointPair >& outPointPairs );

        /**
        \brief Destructor
         */
        ~ChppLosCollisionAvoider();


    private:
        void storeCentipedeObject ( LosCentipedeLinkGeometry& inCentipedeObj );

        std::vector<LosCentipedeLinkGeometry*> attCentipedeObjects;
        std::vector< std::pair<LosRobotCylinder*, LosHalfspace*> > attRCFH;
        std::vector< std::pair<LosRobotCylinder*, LosCylinder*> > attRCFC;
        std::vector< std::pair<LosRobotCylinder*, LosRobotCylinder*> > attRCRC;
        std::vector< std::pair<LosFootprint*, LosHalfspace*> > attFPFH;


        std::vector<ObstacleDamper> attFPFHDampers;
        std::vector<ObstacleDamper> attRCFHDampers;
        std::vector<ObstacleDamper> attRCFCDampers;
        std::vector<SelfDamper> attRCRCDampers;
        std::vector<VelocityDamper*> attDampers;

        void calcLinkConstraint ( ObstacleDamper& outDamper, unsigned int constraintRank );
        void calcObstacleConstraint ( ObstacleDamper& outDamper, unsigned int constraintRank );
        void calcSelfConstraint ( SelfDamper& outDamper, unsigned int constraintRank );

        void crossProd3 ( const vector3d& v1, const vector3d& v2, vector3d& retV );

        matrixNxP tempJac3d,tempJac3dLong,noutM,jac1d;
        
        PointPair PP;
};

#endif
