/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/
#ifndef HPP_LOS_MULTI_TASK_H
#define HPP_LOS_MULTI_TASK_H

#include "hpplocalstepper/tasks/hppLosTask.hh"

/**
A task to project a foot of a CjrlHumanoidDynamicRobot on the ground. This is intended for use as a permanent constraint to ChppCentipede object.
\ingroup tasks
 */
class ChppLosMultiTask: public ChppLosTask
{
    public:
       /**
        \brief Constructor
        */
        ChppLosMultiTask(ChppLosCentipede& inCentipede):ChppLosTask(inCentipede){};
        /**
        \brief Add tasks
         */
        bool addTask(ChppLosTask& inTask)
        {
            if (inTask.differentialSystem().matrix().size2()!=attCentipede->numberDof())
            {
                std::cout << "Bad task entered" <<std::endl;
                return false;
            }
            attTasks.push_back(&inTask);
            attConstraint->matrix().resize(attConstraint->matrix().size1()+inTask.differentialSystem().matrix().size1(),attCentipede->numberDof());
            attConstraint->vector().resize(attConstraint->vector().size()+inTask.differentialSystem().vector().size());
            return true;
        };
        /**
        \brief Compute the linear system to be solved for this task at the current state
         */
        virtual void computeDifferentialSystem() 
        {
            resize();
            unsigned int sz=0;
            for(unsigned int i=0;i<attTasks.size();i++)
            {
                attTasks[i]->computeDifferentialSystem();
                sz += attTasks[i]->differentialSystem().matrix().size1();
            }
            if (sz!=attConstraint->matrix().size1())
            {
                attConstraint->matrix().resize(sz,attCentipede->numberDof());
                attConstraint->vector().resize(sz);
            }
            sz = 0;
            unsigned int szChunck;
            for(unsigned int i=0;i<attTasks.size();i++)
            {
                szChunck = attTasks[i]->differentialSystem().matrix().size1();
                subrange(attConstraint->matrix(),sz,sz+szChunck,0,attCentipede->numberDof()) = attTasks[i]->differentialSystem().matrix();
                subrange(attConstraint->vector(),sz,sz+szChunck) = attTasks[i]->differentialSystem().vector();
                sz +=szChunck;
            }

        };
        /**
        \brief Return the value of the task. The task is strictly solved if the norm of the returned vector is null.
         */
        virtual const vectorN& currentValue() 
        {
            unsigned int sz;
            attValue.resize(0);
            for(unsigned int i=0;i<attTasks.size();i++)
            {
                
                vectorN val = attTasks[i]->currentValue();
                sz = attValue.size();
                attValue.resize(sz+val.size());
                subrange(attValue,sz,sz+val.size()) = val; 
            }
            return attValue;
        };
        /**
        \brief Tell if this task is an equality task
         */
        virtual bool isEqualityTask() const {return true;};
        
    private:
        std::vector<ChppLosTask*> attTasks;
        void equivAsymMat3 ( const vector3d& inVec, matrixNxP& outMat );
        void crossProd3 ( const vector3d& v1, const vector3d& v2, vector3d& retV );
        ChppLosRobotLink* attRobotLink;
        ChppGikStandingRobot* attStandingRobot;
        vectorN tempOmega,tempPos;
        matrixNxP tempRot, jacF;
        matrix3d m3;
        vector3d locVec;
        matrixNxP matOP, matOT,matOTOP;
        vector3d cross;
};

#endif
