#include "hpplocalstepper/hppLosLinkGeometry.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "hpplocalstepper/hppLosLink.hh"
#include "hppGik/robot/hppGikStandingRobot.h"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"


const matrix4d* LosGeometry::associatedTransformation() const
{
    return attFrame;
}

#define nor(x) sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2])
LosCylinder::LosCylinder ( const vector3d& inPointA, const vector3d& inPointB, double inRadius, const matrix4d* refFrame )
{
    attFrame = refFrame;
    locA = A = inPointA;
    locB = B = inPointB;
    locAB = AB = locB - locA;
    attLength = sqrt ( locAB[0]*locAB[0] + locAB[1]*locAB[1] + locAB[2]*locAB[2] );
    attRadius = inRadius;
    //std::cout << "dans LosCylinder "<< absoluteAB() <<  " norme "<< nor(absoluteAB()) << " length " << attLength <<std::endl;
    transform();

}

void LosCylinder::transform()
{
    if ( attFrame )
    {
        m = *attFrame;
        //std::cout << m << std::endl;
        A = m*locA;
        B = m*locB;
        AB = B - A;
    }
}

double LosCylinder::length() const {return attLength;}
double LosCylinder::radius() const {return attRadius;}
const vector3d& LosCylinder::localA() const {return locA;}
const vector3d& LosCylinder::localB() const {return locB;}
const vector3d& LosCylinder::localAB() const {return locAB;}
const vector3d& LosCylinder::absoluteA() const {return A;}
const vector3d& LosCylinder::absoluteB() const {return B;}
const vector3d& LosCylinder::absoluteAB() const {return AB;}


LosHalfspace::LosHalfspace ( const vector3d& inPoint, const vector3d& inOuterNormalN, const matrix4d* refFrame )
{
    attFrame = refFrame;
    attPoint = attAbsPoint = inPoint;
    attNormal = attAbsNormal = inOuterNormalN;
    transform();
}

void LosHalfspace::transform()
{
    if ( attFrame )
    {
        m = *attFrame;
        attAbsPoint = m*attPoint;
        attAbsNormal = m*attNormal;
    }
}

const vector3d& LosHalfspace::borderPoint() const {return attAbsPoint;}
const vector3d& LosHalfspace::outerNormal() const {return attAbsNormal;}



/////////////////////////////////////

LosCentipedeLinkGeometry::LosCentipedeLinkGeometry ( ChppLosCentipede& inCentipede, ChppLosLink& inLink )
{
    attCentipede = &inCentipede;
    attLink = &inLink;
}
ChppLosCentipede& LosCentipedeLinkGeometry::centipede() {return *attCentipede;}
ChppLosLink& LosCentipedeLinkGeometry::link() {return *attLink;}

////////////////////////////////////////

LosRobotLinkGeometry::LosRobotLinkGeometry ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inLink, CjrlJoint& jointJ ) :LosCentipedeLinkGeometry ( inCentipede,inLink )
{
    attJoint = &jointJ;
    attRobotLink = &inLink;
}

ChppLosRobotLink& LosRobotLinkGeometry::robotlink() {return *attRobotLink;}
CjrlJoint& LosRobotLinkGeometry::joint() {return *attJoint;}


///////////////////////////////////////////



LosRobotCylinder::LosRobotCylinder ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inLink, CjrlJoint& jointJ, const vector3d& pointAinJ, const vector3d& pointBinJ,  double inRadius ) :LosRobotLinkGeometry(inCentipede,inLink,jointJ),
LosCylinder ( pointAinJ, pointBinJ, inRadius, &jointJ.currentTransformation() )
{
}



LosFootprint::LosFootprint ( ChppLosCentipede& inCentipede, ChppLosLink& inLink, const ChppLosFootprintShape& inShape ) :LosCentipedeLinkGeometry ( inCentipede, inLink )
{
    attCentipede = &inCentipede;
    attLink = &inLink;
    lshp = &inShape;
    vector3d v ( 0,0,0 );
    attVertices.push_back ( v );
    attVertices.push_back ( v );
    attVertices.push_back ( v );
    attVertices.push_back ( v );
    transform();
}
const ChppLosFootprintShape& LosFootprint::localshape() const
{
    return *lshp;
}
const std::vector<vector3d>& LosFootprint::vertices() const
{
    return attVertices;
}
void LosFootprint::transform()
{
    const ChppLosFootprint& fp = attLink->frontprint();
    attVertices[0][0] = fp.x + fp.UXx*lshp->xUpRight + fp.UYx*lshp->yUpRight;
    attVertices[0][1] = fp.y + fp.UXy*lshp->xUpRight + fp.UYy*lshp->yUpRight;

    attVertices[1][0] = fp.x + fp.UXx*lshp->xDownRight + fp.UYx*lshp->yDownRight;
    attVertices[1][1] = fp.y + fp.UXy*lshp->xDownRight + fp.UYy*lshp->yDownRight ;

    attVertices[2][0] = fp.x + fp.UXx*lshp->xDownLeft + fp.UYx*lshp->yDownLeft;
    attVertices[2][1] = fp.y + fp.UXy*lshp->xDownLeft + fp.UYy*lshp->yDownLeft;

    attVertices[3][0] = fp.x + fp.UXx*lshp->xUpLeft + fp.UYx*lshp->yUpLeft;
    attVertices[3][1] = fp.y + fp.UXy*lshp->xUpLeft + fp.UYy*lshp->yUpLeft;
}


