#include "boost/numeric/ublas/vector_proxy.hpp"
#include "boost/numeric/ublas/matrix_proxy.hpp"
#include "hpplocalstepper/tasks/hppLosConfigurationTask.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "hpplocalstepper/hppLosLink.hh"
#include "hpphik/hppHikLinearSystem.hh"
#include "hppGik/robot/hppGikStandingRobot.h"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"

using namespace boost::numeric::ublas;

ChppLosConfigurationTask::ChppLosConfigurationTask ( ChppLosCentipede& inCentipede, const vectorN& inTarget, const vectorN& inMask ) :ChppLosTask ( inCentipede )
{
    attTarget = inTarget;
    attMask = inMask;
    

    if ( ( inTarget.size() == inCentipede.numberDof() ) && ( inMask.size() == inCentipede.numberDof() ) )
    {
        attTarget = inTarget;
        attMask = inMask;
    }
    else
    {
        std::cout << "ChppLosConfigurationTask::constructor bad vector sizes" <<std::endl;
        inCentipede.getConfiguration(attTarget);
        attMask = scalar_vector<double> ( inCentipede.numberDof(), 1 );
    }

    
    unsigned int h = 0;
    for ( unsigned int i = 0; i< inCentipede.numberDof();i++ )
        if ( attMask ( i ) == 1 )
            h++;


    attConstraint->matrix().resize ( h,inCentipede.numberDof() );
    attConstraint->matrix().clear();
    attConstraint->vector().resize ( h );
    attValue.resize(h);
    
    attConfig.resize(inCentipede.numberDof());

    unsigned int line = 0;

    for ( unsigned int col = 0; col< inCentipede.numberDof();col++ )
    {
        if ( attMask ( col ) == 1 )
        {
            attConstraint->matrix() ( line, col ) = 1;
            line++;
        }
    }
    
    attMaxVelocity = 1e10;
}


void ChppLosConfigurationTask::maxVelocityNorm ( double inMV )
{
    attMaxVelocity = (inMV>0.0)?inMV:0.0;
}

const vectorN& ChppLosConfigurationTask::targetConfiguration()
{
    return attTarget;
}

bool ChppLosConfigurationTask::targetConfiguration ( const vectorN& inV )
{
    if ( inV.size() != attCentipede->numberDof() )
    {
        std::cout << "ChppLosConfigurationTask::mask bad vector size" <<std::endl;
        return false;
    }
    attTarget = inV;
    return true;
}

const vectorN& ChppLosConfigurationTask::mask()
{
    return attMask;
}

bool ChppLosConfigurationTask::mask ( const vectorN& inV )
{
    if ( inV.size() != attCentipede->numberDof() )
    {
        std::cout << "ChppLosConfigurationTask::mask bad vector size" <<std::endl;
        return false;
    }
    attMask = inV;
    return true;
}

void ChppLosConfigurationTask::computeDifferentialSystem()
{
    unsigned int line = 0;
    double max = 0;
    attCentipede->getConfiguration(attConfig);
    for ( unsigned int col = 0; col< attCentipede->numberDof();col++ )
    {
        if ( attMask ( col ) == 1 )
        {
            attConstraint->vector() ( line ) = attTarget(col) - attConfig(col);
            if (fabs(attConstraint->vector() ( line ))>max)
                max = fabs(attConstraint->vector() ( line ));
            line++;
        }
    }
    if (max > attMaxVelocity)
        attConstraint->vector() = attMaxVelocity*attConstraint->vector()/max;
    
}

const vectorN& ChppLosConfigurationTask::currentValue()
{
    unsigned int line = 0;
    attCentipede->getConfiguration(attConfig);
    for ( unsigned int col = 0; col< attCentipede->numberDof();col++ )
    {
        if ( attMask ( col ) == 1 )
        {
            attValue ( line ) = attTarget(col) - attConfig(col);
            line++;
        }
    }
    return attValue;
}

