#include "hpplocalstepper/tasks/hppLosFeetAntiCollisionTask.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "hpplocalstepper/hppLosLink.hh"
#include "hpphik/hppHikLinearSystem.hh"
#include "hppGik/robot/hppGikStandingRobot.h"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"
ChppLosFeetAntiCollisionTask::ChppLosFeetAntiCollisionTask ( ChppLosCentipede& inCentipede, ChppLosLink& inLink ) :ChppLosTask ( inCentipede )
{
    attLink = &inLink;

    attConstraint->vector().resize ( 4 );
    attConstraint->matrix().resize ( 4, attCentipede->numberDof() );
    attValue.resize ( 4 );
    attConstraint->matrix().clear();
    attConstraint->vector().clear();
    attStepXi = 0.02;
    attStepDi = 0.05;
    attStepDs = 0.02;
}


void ChppLosFeetAntiCollisionTask::setMargins ( double inDi, double inDs, double inXi )
{
    attStepXi = inXi;
    attStepDs = inDs;
    attStepDi = inDi;
}

void ChppLosFeetAntiCollisionTask::computeDifferentialSystem()
{
    resize();
    {
        const ChppLosFootprintArea& footprintArea = attLink->leftStepArea();
        const ChppLosJacobianPack& jp = attLink->jacobianPack();
        unsigned int startRank = attLink->rank();

        double coord = jp.ULinRy;
        if ( coord < footprintArea.minY + attStepDi )
            attConstraint->vector() ( 0 ) = attStepXi* ( coord - footprintArea.minY - attStepDs ) / ( attStepDi - attStepDs );
        else
            attConstraint->vector() ( 0 ) = attStepXi;

        for ( unsigned int i=0;i<attLink->numberDof();i++ )
            attConstraint->matrix() ( 0,startRank+i ) = -jp.jacLUinR ( 1,i );

        coord = jp.DLinRy;
        if ( coord < footprintArea.minY + attStepDi )
            attConstraint->vector() ( 1 ) = attStepXi* ( coord - footprintArea.minY - attStepDs ) / ( attStepDi - attStepDs );
        else
            attConstraint->vector() ( 1 ) = attStepXi;

        for ( unsigned int i=0;i<attLink->numberDof();i++ )
            attConstraint->matrix() ( 1,startRank+i ) = -jp.jacLDinR ( 1,i );
    }

    {
        const ChppLosFootprintArea& footprintArea = attLink->rightStepArea();
        const ChppLosJacobianPack& jp = attLink->jacobianPack();
        unsigned int startRank = attLink->rank();

        double coord = jp.URinLy;
        if ( coord > footprintArea.maxY - attStepDi )
            attConstraint->vector() ( 2 ) = attStepXi* ( coord - footprintArea.maxY + attStepDs ) / ( attStepDs - attStepDi );
        else
            attConstraint->vector() ( 2 ) = attStepXi;

        for ( unsigned int i=0;i<attLink->numberDof();i++ )
            attConstraint->matrix() ( 2,startRank+i ) = jp.jacRUinL ( 1,i );

        coord = jp.DRinLy;
        if ( coord > footprintArea.maxY - attStepDi )
            attConstraint->vector() ( 3 ) = attStepXi* ( coord - footprintArea.maxY + attStepDs ) / ( attStepDs - attStepDi );
        else
            attConstraint->vector() ( 3 ) = attStepXi;

        for ( unsigned int i=0;i<attLink->numberDof();i++ )
            attConstraint->matrix() ( 3,startRank+i ) = jp.jacRDinL ( 1,i );
    }
}

const vectorN& ChppLosFeetAntiCollisionTask::currentValue()
{
    const ChppLosJacobianPack& jp = attLink->jacobianPack();
    attValue.clear();

    {
        const ChppLosFootprintArea& fa = attLink->leftStepArea();

        if ( jp.ULinRy < fa.minY + attStepDs )
            attValue ( 0 ) = fa.minY + attStepDs - jp.ULinRy;

        if ( jp.DLinRy < fa.minY + attStepDs )
            attValue ( 1 ) = fa.minY + attStepDs - jp.DLinRy;
    }

    {
        const ChppLosFootprintArea& fa = attLink->rightStepArea();

        if ( jp.URinLy > fa.maxY - attStepDs )
            attValue ( 2 ) = jp.URinLy - fa.maxY + attStepDs;

        if ( jp.DRinLy > fa.maxY - attStepDs )
            attValue ( 3 ) = jp.DRinLy - fa.maxY + attStepDs ;
    }

    return attValue;

}


