#include "boost/numeric/ublas/vector_proxy.hpp"
#include "boost/numeric/ublas/matrix_proxy.hpp"
#include "hpplocalstepper/tasks/hppLosLinkConfigurationTask.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "hpplocalstepper/hppLosLink.hh"
#include "hpphik/hppHikLinearSystem.hh"
#include "hppGik/robot/hppGikStandingRobot.h"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"

using namespace boost::numeric::ublas;

ChppLosLinkConfigurationTask::ChppLosLinkConfigurationTask ( ChppLosCentipede& inCentipede, ChppLosLink& inlink, const vectorN& inTarget, const vectorN& inMask ) :ChppLosTask ( inCentipede )
{
    attLink = &inlink;
    attTarget = inTarget;
    attMask = inMask;

    if ( ( inTarget.size() == attLink->numberDof() ) && ( inMask.size() == attLink->numberDof() ) )
    {
        attTarget = inTarget;
        attMask = inMask;
    }
    else
    {
        std::cout << "ChppLosLinkConfigurationTask::constructor bad vector sizes" <<std::endl;
        attTarget = attLink->currentConfiguration();
        attMask = scalar_vector<double> ( attLink->numberDof(), 1 );
    }

    unsigned int h = 0;
    for ( unsigned int i = 0; i< attLink->numberDof();i++ )
        if ( attMask ( i ) == 1 )
            h++;


    attConstraint->matrix().resize ( h,inCentipede.numberDof() );
    attConstraint->matrix().clear();
    attConstraint->vector().resize ( h );
    attValue.resize ( h );

    unsigned int line = 0;

    for ( unsigned int col = 0; col<  attLink->numberDof();col++ )
    {
        if ( attMask ( col ) == 1 )
        {
            attConstraint->matrix() ( line, attLink->rank()+col ) = 1;
            line++;
        }
    }

    attMaxVelocity = 1e10;
}

bool ChppLosLinkConfigurationTask::resize()
{
    if ( ChppLosTask::resize() )
    {
        unsigned int line = 0;

        for ( unsigned int col = 0; col< attLink->numberDof();col++ )
        {
            if ( attMask ( col ) == 1 )
            {
                attConstraint->matrix() ( line, attLink->rank()+col ) = 1;
                line++;
            }
        }
        return true;
    }
    return false;
}

void ChppLosLinkConfigurationTask::maxVelocityNorm ( double inMV )
{
    attMaxVelocity = ( inMV>0.0 ) ?inMV:0.0;
}

const vectorN& ChppLosLinkConfigurationTask::targetConfiguration()
{
    return attTarget;
}

bool ChppLosLinkConfigurationTask::targetConfiguration ( const vectorN& inV )
{
    if ( inV.size() != attLink->numberDof() )
    {
        std::cout << "ChppLosLinkConfigurationTask::mask bad vector size" <<std::endl;
        return false;
    }
    attTarget = inV;
    return true;
}

const vectorN& ChppLosLinkConfigurationTask::mask()
{
    return attMask;
}

bool ChppLosLinkConfigurationTask::mask ( const vectorN& inV )
{
    if ( inV.size() != attCentipede->numberDof() )
    {
        std::cout << "ChppLosLinkConfigurationTask::mask bad vector size" <<std::endl;
        return false;
    }
    attMask = inV;
    return true;
}

void ChppLosLinkConfigurationTask::computeDifferentialSystem()
{
    resize();
    unsigned int line = 0;
    double max = 0;
    for ( unsigned int col = 0; col< attLink->numberDof();col++ )
    {
        if ( attMask ( col ) == 1 )
        {
            attConstraint->vector() ( line ) = attTarget ( col ) - attLink->currentConfiguration() ( col );
            if ( fabs ( attConstraint->vector() ( line ) ) >max )
                max = fabs ( attConstraint->vector() ( line ) );
            line++;
        }
    }
    if ( max > attMaxVelocity )
        attConstraint->vector() = attMaxVelocity*attConstraint->vector() /max;

}

const vectorN& ChppLosLinkConfigurationTask::currentValue()
{
    unsigned int line = 0;

    for ( unsigned int col = 0; col< attLink->numberDof();col++ )
    {
        if ( attMask ( col ) == 1 )
        {
            attValue ( line ) = attTarget ( col ) - attLink->currentConfiguration() ( col );
            line++;
        }
    }
    return attValue;
}

