#include "hpplocalstepper/tasks/hppLosTask.hh"
#include "hpphik/hppHikLinearSystem.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"

ChppLosTask::ChppLosTask ( ChppLosCentipede& inCentipede )
{
    attCentipede = &inCentipede;
    attConstraint = new ChppHikLinearSystem ( attCentipede->numberDof() );
    attDamp = 0.0;
    attEndTime = 0.0;
    attStartTime = 0.0;
}

ChppLosTask::~ChppLosTask() {delete attConstraint;}

ChppHikLinearSystem& ChppLosTask::differentialSystem() {return *attConstraint;}

double ChppLosTask::dampingFactor() {return attDamp;}

void ChppLosTask::dampingFactor ( double inF ) {attDamp = inF;}

void ChppLosTask::startTime ( double inT ) {attStartTime = inT;}

double ChppLosTask::startTime() {return attStartTime;}

void ChppLosTask::endTime ( double inT ) {attEndTime = inT;}

double ChppLosTask::endTime() {return attEndTime;}

bool ChppLosTask::resize()
{
    if ( attConstraint->matrix().size2() != attCentipede->numberDof() )
    {
        attConstraint->matrix().resize ( attConstraint->matrix().size1(),attCentipede->numberDof() );
        attConstraint->matrix().clear();
        return true;
    }
    return false;
}

ChppLosTaskOnJoint::ChppLosTaskOnJoint ( ChppLosCentipede& inCentipede,CjrlJoint& inJoint ) :ChppLosTask ( inCentipede )
{
    attJoint = &inJoint;
}

CjrlJoint* ChppLosTaskOnJoint::joint()
{
    return attJoint;
}

void ChppLosTaskOnJoint::joint ( CjrlJoint* inJoint )
{
    attJoint = inJoint;
}

