#include "boost/numeric/ublas/vector_proxy.hpp"
#include "boost/numeric/ublas/matrix_proxy.hpp"
#include "hpplocalstepper/tasks/hppLosTransformationTask.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "hpplocalstepper/hppLosLink.hh"
#include "hpphik/hppHikLinearSystem.hh"
#include "hppGikTools.h"
#include "hppGik/robot/hppGikStandingRobot.h"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"

using namespace boost::numeric::ublas;

ChppLosTransformationTask::ChppLosTransformationTask (ChppLosCentipede& inCentipede, ChppLosRobotLink& inRobotLink, CjrlJoint& inJoint,const vector3d& inLocalPoint,const matrix4d& inWorlTarget ):ChppLosTaskOnJoint(inCentipede,inJoint)
{
    attRobotLink = &inRobotLink;
    attLocalPoint = inLocalPoint;
    attTargetTransform = inWorlTarget;

    attConstraint->matrix().resize(6,attCentipede->numberDof());
    attConstraint->matrix().clear();
    attConstraint->vector().resize(6);
    attValue.resize(6);
    attMaxTVelocity = 1e10;
    attMaxRVelocity = 1e10;
    
    tempOmega.resize(3);
    CurPos.resize(3);
    CurRot.resize(3,3);
    TarPos.resize(3);
    TarRot.resize(3,3);
    tempGapRot.resize(3,3);
    tempGap.resize(3);
}


const vector3d& ChppLosTransformationTask::localPoint()
{
    return attLocalPoint;
}
void ChppLosTransformationTask::localPoint ( const vector3d& inVec )
{
    attLocalPoint = inVec;
}

const matrix4d& ChppLosTransformationTask::targetTransformation()
{
    return attTargetTransform;
}

void ChppLosTransformationTask::targetTransformation ( const matrix4d& inM )
{
    attTargetTransform = inM;
}

void ChppLosTransformationTask::maxTranslationVelocity(double inMV)
{
    attMaxTVelocity = (inMV>0.0)?inMV:0.0;
}

void ChppLosTransformationTask::maxRotationVelocity(double inMV)
{
    attMaxRVelocity = (inMV>0.0)?inMV:0.0;
}

const vectorN& ChppLosTransformationTask::currentValue()
{
    //std::cout << "attJoint->currentTransformation() "<< attJoint->currentTransformation() << std::endl;
    //std::cout <<  "attTargetTransform "<< attTargetTransform << std::endl;
    ChppGikTools::HtoRT ( attJoint->currentTransformation(),CurRot,CurPos );
    ChppGikTools::HtoRT ( attTargetTransform,TarRot,TarPos );
    
    m = attJoint->currentTransformation();
    attCurrentPoint =  m * attLocalPoint;
    attValue(0) = TarPos(0) - attCurrentPoint(0);
    attValue(1) = TarPos(1) - attCurrentPoint(1);
    attValue(2) = TarPos(2) - attCurrentPoint(2);
    
    noalias(tempGapRot) =  prod(trans(CurRot),TarRot);
    ChppGikTools::RottoOmega ( tempGapRot,tempOmega );
    noalias (TarPos) = prod(CurRot, tempOmega);
    
    attValue(3) = TarPos[0];
    attValue(4) = TarPos[1];
    attValue(5) = TarPos[2];

    return attValue;
}

void ChppLosTransformationTask::computeDifferentialSystem()
{
    resize();
    ChppGikTools::HtoRT ( attJoint->currentTransformation(),CurRot,CurPos );
    ChppGikTools::HtoRT ( attTargetTransform,TarRot,TarPos );
    
    m =attJoint->currentTransformation();
    attCurrentPoint = m * attLocalPoint;
    tempGap(0) = TarPos[0] - attCurrentPoint[0];
    tempGap(1) = TarPos[1] - attCurrentPoint[1];
    tempGap(2) = TarPos[2] - attCurrentPoint[2];

    noalias(tempGapRot) =  prod(trans(CurRot),TarRot);
    ChppGikTools::RottoOmega ( tempGapRot,tempOmega );
    noalias (TarPos) = prod(CurRot, tempOmega);

    double nor = norm_2(tempGap);
    if (nor > attMaxTVelocity)
        tempGap = attMaxTVelocity*tempGap/nor;
    nor = norm_2(TarPos);
    if (nor > attMaxRVelocity)
        TarPos = attMaxRVelocity*TarPos/nor;
    
    noalias(subrange(attConstraint->vector(),0,3)) = tempGap;
    noalias(subrange(attConstraint->vector(),3,6)) = TarPos;
    
    attConstraint->matrix().clear();
    attRobotLink->standingRobot().robot()->getJacobian ( *attRobotLink->rootJoint(),*attJoint, attLocalPoint, attConstraint->matrix(), attRobotLink->rank(), false );
    ChppLosLink* plink = attRobotLink->parent();
    while ( plink!=0 )
    {
        plink->getJacobian ( attCurrentPoint,attConstraint->matrix(),plink->rank() );
        plink = plink->parent();
    }
}

