#include "HRP2Wrapper.hh"
#include "hpplocalstepper/hppLosLinkGeometry.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "hpplocalstepper/hppLosLink.hh"
#include "hppGik/robot/hppGikStandingRobot.h"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"
#include <string.h>

#define M4_IJ MAL_S4x4_MATRIX_ACCESS_I_J

HRP2Wrapper::HRP2Wrapper ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inLink )
{
    attCentipede = &inCentipede;
    attLink = &inLink;

    CjrlHumanoidDynamicRobot& inRobot = *attLink->standingRobot().robot();

    CjrlJoint* joint = inRobot.rightWrist();
    attHandR = new LosRobotCylinder( inCentipede, inLink, *joint, vector3d(0,0,0.02), vector3d(0,0,0.155), 0.06);
    attCylinders.push_back(attHandR);
    
    joint = joint->parentJoint()->parentJoint();
    attLowerarmR = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(-0.01,0,0), vector3d(0,0,0.23), 0.06);
    attCylinders.push_back(attLowerarmR);
    
    joint = joint->parentJoint();
    attUpperarmR = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0,0,0), vector3d(-0.25,0,-0.01), 0.065);
    attCylinders.push_back(attUpperarmR);
    
    attShoulderR = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0.005,-0.05,0.006), vector3d(0.005,0.05,0.006), 0.07);
    attCylinders.push_back(attShoulderR);
    
    joint = inRobot.leftWrist();
    attHandL = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0,0,0.02), vector3d(0,0,0.155), 0.06);
    attCylinders.push_back(attHandL);
    
    joint = joint->parentJoint()->parentJoint();
    attLowerarmL = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0.01,0,0), vector3d(0,0,0.23), 0.06);
    attCylinders.push_back(attLowerarmL);
    
    joint = joint->parentJoint();
    attUpperarmL = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0,0,0), vector3d(-0.25,0,0.01), 0.065);
    attCylinders.push_back(attUpperarmL);
    
    attShoulderL = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0.005,-0.05,-0.006), vector3d(0.005,0.05,-0.006), 0.07);
    attCylinders.push_back(attShoulderL);
    
    joint = inRobot.waist();
    attHips = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(-0.03,0,0.2), vector3d(-0.03,0,0.225), 0.2);
    //attHips = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(-0.02,0.005,0.20), vector3d(-0.02,-0.005,0.20), 0.205);
    attCylinders.push_back(attHips);

    joint = inRobot.gazeJoint()->parentJoint()->parentJoint();
    //attChest = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0,-0.045,-0.1), vector3d(0,-0.045,-0.15), 0.225);
    attChest = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0.08,-0.05,-0.11), vector3d(-0.08,-0.05,-0.115), 0.195);
    attCylinders.push_back(attChest);

    joint = inRobot.rightAnkle()->parentJoint()->parentJoint();
    attLowerlegR = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(-0.06,0.04,-0.03), vector3d(-0.044473,-0.007,0.305),  0.085);
    attCylinders.push_back(attLowerlegR);

    joint = joint->parentJoint();
    attUpperlegR = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(-0.074,0.0,0.02), vector3d(-0.0622,0.01,0.289),0.077);
    attCylinders.push_back(attUpperlegR);

    joint = inRobot.leftAnkle()->parentJoint()->parentJoint();
    attLowerlegL = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0.06,0.04,-0.03), vector3d(0.044473,-0.007,0.305),  0.085);
    attCylinders.push_back(attLowerlegL);

    joint = joint->parentJoint();
    attUpperlegL = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0.074,0.0,0.02), vector3d(0.0622,0.01,0.289), 0.077);
    attCylinders.push_back(attUpperlegL);

    joint = inRobot.gazeJoint();
    attHead = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(0.0,0.0,-0.07), vector3d(0.0,0.0,-0.08), 0.165);
    attCylinders.push_back(attHead);
    
    joint = inRobot.rightWrist();
    attOpenhandR = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(-0.02,0,0.14), vector3d(0.01,0,0.14), 0.08);
    attCylinders.push_back(attOpenhandR);
    
    joint = inRobot.leftWrist();
    attOpenhandL = new LosRobotCylinder(inCentipede, inLink, *joint, vector3d(-0.01,0,0.14), vector3d(0.02,0,0.14),0.08);
    attCylinders.push_back(attOpenhandL);
}

void HRP2Wrapper::updateModel()
{
    for ( unsigned int i = 0;i<attCylinders.size();i++ )
        attCylinders[i]->transform();
}

HRP2Wrapper::~HRP2Wrapper()
{
    for ( unsigned int i = 0;i<attCylinders.size();i++ )
        delete attCylinders[i];
}
