#include "dynamicsJRLJapan/dynamicsJRLJapanFactory.h"
#include "hrp2Dynamics/hrp2OptHumanoidDynamicRobot.h"


#include "hppLocalStepperTest.h"
#include "hpplocalstepper/hppLosCentipede.hh"


#include "hppGik/robot/hppRobotMotion.h"


#include "hpplocalstepper/tasks/hppLosCollisionAvoider.hh"
#include "hpplocalstepper/tasks/hppLosComBoundingTask.hh"
#include "hpplocalstepper/tasks/hppLosComTask.hh"
#include "hpplocalstepper/tasks/hppLosFeetProjectionTask.hh"
#include "hpplocalstepper/tasks/hppLosFeetAntiCollisionTask.hh"
#include "hpplocalstepper/tasks/hppLosFeetAreaTask.hh"
#include "hpplocalstepper/tasks/hppLosDiscAvoider.hh"
#include "hpplocalstepper/tasks/hppLosPositionTask.hh"
#include "hpplocalstepper/tasks/hppLosParallelTask.hh"
#include "hpplocalstepper/tasks/hppLosPlaneTask.hh"
#include "hpplocalstepper/tasks/hppLosCoplanarTask.hh"
#include "hpplocalstepper/tasks/hppLosLinkConfigurationTask.hh"
#include "hpplocalstepper/tasks/hppLosGazeTask.hh"
#include "hpplocalstepper/hppLosSolver.hh"
#include "hpplocalstepper/hppLosCentipedeFactory.hh"
#include "hpplocalstepper/hppLosCentipede.hh"
#include "hpplocalstepper/hppLosLink.hh"
#include "hpplocalstepper/hppLosLinkGeometry.hh"
#include "hppGik/robot/hppGikStandingRobot.h"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"
#include "hppGik/tasks/hppGikHandTask.h"
#include "HRP2Wrapper.hh"
#include "hppLosMotionGenerator.h"

#define M3_IJ MAL_S3x3_MATRIX_ACCESS_I_J
#define M4_IJ MAL_S4x4_MATRIX_ACCESS_I_J

using namespace boost::numeric::ublas;

ChppGikStandingRobot* ChppLocalStepperTest::createHumanoidRobot ( const std::string& inPath )
{
    dynamicsJRLJapan::ObjectFactory jrlRobotFactory;

    CjrlHumanoidDynamicRobot* attRobot = new Chrp2OptHumanoidDynamicRobot ( &jrlRobotFactory );
    std::string linkJointFname,specificitiesFile;
    linkJointFname = inPath + "/HRP2LinkJointRank.xml";
    specificitiesFile = inPath + "/HRP2Specificities.xml";
    std::string modelname = inPath + "/HRP2.wrl";

    dynamicsJRLJapan::parseOpenHRPVRMLFile ( *attRobot, modelname,linkJointFname,specificitiesFile );

    std::string property,value;
    property="ComputeZMP"; value="false";attRobot->setProperty ( property,value );
    property="TimeStep"; value="0.005";attRobot->setProperty ( property,value );
    property="ComputeAccelerationCoM"; value="false";attRobot->setProperty ( property,value );
    property="ComputeBackwardDynamics"; value="false";attRobot->setProperty ( property,value );
    property="ComputeMomentum"; value="false";attRobot->setProperty ( property,value );
    property="ComputeAcceleration"; value="false";attRobot->setProperty ( property,value );
    property="ComputeVelocity"; value="false";attRobot->setProperty ( property,value );
    property="ComputeSkewCoM"; value="false";attRobot->setProperty ( property,value );
    property="ComputeCoM"; value="true";attRobot->setProperty ( property,value );

    unsigned int nDof = attRobot->numberDof();
    vectorN halfsittingConf ( nDof );

    //Half sitting
    double dInitPos[40] =
    {
        0.0, 0.0, -26.0, 50.0, -24.0, 0.0, // right leg
        0.0, 0.0, -26.0, 50.0, -24.0, 0.0, // left leg
        0.0, 0.0, // chest
        0.0, 0.0, // head
        15.0, -10.0, 0.0, -30.0, 0.0, 0.0, // right arm
        10.0, // right hand clench
        15.0,  10.0, 0.0, -30.0, 0.0, 0.0, // left arm
        10.0, // left hand clench
        -10.0, 10.0, -10.0, 10.0, -10.0, // right hand parallel mechanism
        -10.0, 10.0, -10.0, 10.0, -10.0  // left hand parallel mechanism
    };

    //waist x y z
    halfsittingConf ( 0 ) = 0.0;
    halfsittingConf ( 1 ) = 0.0;
    halfsittingConf ( 2 ) = 0.6487;
    //waist roll pitch yaw
    halfsittingConf ( 3 ) = 0.0;
    halfsittingConf ( 4 ) = 0.0;
    halfsittingConf ( 5 ) = 0.0;

    //joints
    for ( unsigned int i=6;i<nDof;i++ )
        halfsittingConf ( i ) = dInitPos[i-6]*M_PI/180;

    zero_vector<double> zeros ( attRobot->numberDof() );
    attRobot->currentConfiguration ( halfsittingConf );
    attRobot->currentVelocity ( zeros );
    attRobot->currentAcceleration ( zeros );
    attRobot->computeForwardKinematics();

    //set gaze origin and direction
    vector3d gazeDir,gazeOrigin;
    gazeDir[0] = 0;
    gazeDir[1] = cos ( 10*M_PI/180 );
    gazeDir[2] = sin ( 10*M_PI/180 );

    gazeOrigin[0] = 0;
    gazeOrigin[1] = 0;
    gazeOrigin[2] = -0.118;

    attRobot->gaze ( gazeDir, gazeOrigin );

    ChppGik2DShape leftFootShape, rightFootShape;

    ChppGik2DVertex vert;

    vert.x = 0.135;
    vert.y = 0.079;
    leftFootShape.vertices.push_back ( vert );
    vert.x = -0.105;
    leftFootShape.vertices.push_back ( vert );
    vert.y = -0.059;
    leftFootShape.vertices.push_back ( vert );
    vert.x = 0.135;
    leftFootShape.vertices.push_back ( vert );

    vert.x = 0.135;
    vert.y = 0.059;
    rightFootShape.vertices.push_back ( vert );
    vert.x = -0.105;
    rightFootShape.vertices.push_back ( vert );
    vert.y = -0.079;
    rightFootShape.vertices.push_back ( vert );
    vert.x = 0.135;
    rightFootShape.vertices.push_back ( vert );


    ChppGikStandingRobot* standingRobot = new ChppGikStandingRobot ( *attRobot, leftFootShape, rightFootShape );


    standingRobot->staticState ( halfsittingConf );

    return standingRobot;
}

ChppLocalStepperTest::ChppLocalStepperTest ( const std::string& inPath )
{
    createCentipede ( inPath );
    attpath = inPath;
}

void ChppLocalStepperTest::createCentipede ( const std::string& inPath )
{

    ChppGikStandingRobot* sr1 = createHumanoidRobot ( inPath );
    standingRobots.push_back ( sr1 );
    ChppLosFootprint iLeft,iRight;
    matrix4d mf = sr1->robot()->rightAnkle()->currentTransformation();
    iRight.x = M4_IJ ( mf,0,3 );
    iRight.y =  M4_IJ ( mf,1,3 );
    iRight.th =  atan2 ( M4_IJ ( mf,1,0 ),M4_IJ ( mf,0,0 ) );

    mf = sr1->robot()->leftAnkle()->currentTransformation();
    iLeft.x = M4_IJ ( mf,0,3 );
    iLeft.y =  M4_IJ ( mf,1,3 );
    iLeft.th =  atan2 ( M4_IJ ( mf,1,0 ),M4_IJ ( mf,0,0 ) );

    ChppLosFootprintShape lshape;
    lshape.xDownLeft = -0.105;
    lshape.xDownRight = -0.105;
    lshape.xUpRight = 0.135;
    lshape.xUpLeft = 0.135;
    lshape.yDownLeft = 0.079;
    lshape.yUpLeft = 0.079;
    lshape.yDownRight = -0.059;
    lshape.yUpRight = -0.059;

    ChppLosFootprintArea larea ;
    larea.maxX = 0.22;
    larea.minX = -0.22;
    larea.maxY = 0.25;
    larea.minY = 0.07;
    larea.maxTheta = M_PI/4;
    larea.minTheta = -0.1;

    ChppLosFootprintShape rshape;
    rshape.xDownLeft = -0.105;
    rshape.xDownRight = -0.105;
    rshape.xUpRight = 0.135;
    rshape.xUpLeft = 0.135;
    rshape.yDownLeft = 0.059;
    rshape.yUpLeft = 0.059;
    rshape.yDownRight = -0.079;
    rshape.yUpRight = -0.079;

    ChppLosFootprintArea rarea ;
    rarea.maxX = larea.maxX;
    rarea.minX = larea.minX;
    rarea.maxY = -larea.minY;
    rarea.minY = -larea.maxY;
    rarea.maxTheta = -larea.minTheta;
    rarea.minTheta = -larea.maxTheta;

    unsigned int nSteps = 2;
    
    attCentipede = ChppLosCentipedeFactory::createSpringman ( iLeft,iRight, *sr1, nSteps, false, larea,  rarea, lshape, rshape, attCentipedeParts, attMask );
    centipedes.push_back ( attCentipede );
}


ChppLocalStepperTest::~ChppLocalStepperTest()
{
    for ( unsigned int i=0;i<standingRobots.size();i++ )
        delete standingRobots[i];

    for ( unsigned int i=0;i<centipedes.size();i++ )
        delete centipedes[i];

    std::list<ChppLosLink*>::iterator iter = attCentipedeParts.begin();
    while (iter != attCentipedeParts.end())
    {
        delete *iter;
        iter++;
    }
}


void ChppLocalStepperTest::plan()
{
    std::vector<ChppLosRobotLink*> robotslinks;
    std::vector<CjrlHumanoidDynamicRobot*> robots;
    std::list<ChppLosRobotLink*>::iterator iterRobot;
    std::vector<ChppLosTask*> tasks,constraints;
    std::list<ChppLosTask*> centipedeConstraints;

    vectorN hsc = subrange ( standingRobots[0]->halfsittingConfiguration(),6,46 );
    vectorN configMask = subrange ( standingRobots[0]->maskFactory()->wholeBodyMask(),6,46 );

    for ( iterRobot = attCentipede->robotLinks().begin(); iterRobot!=attCentipede->robotLinks().end(); iterRobot++ )
    {
        robotslinks.push_back ( *iterRobot );
        robots.push_back ( ( *iterRobot )->standingRobot().robot() );
    }

    //Create solver
    ChppLosSolver solver1 ( *attCentipede );
    //std::cout << robots[0]->upperBoundDof(robots[0]->gazeJoint()->parentJoint()->parentJoint()->rankInConfiguration()) <<std::endl;

    //std::cout << robots[0]->lowerBoundDof(robots[0]->gazeJoint()->parentJoint()->parentJoint()->rankInConfiguration()) <<std::endl;

    //Limit the chest 1 dof to avoid collision
    robotslinks[0]->upperDofLimit ( robots[0]->gazeJoint()->parentJoint()->parentJoint()->rankInConfiguration()-6, 0.8 );
    /*
    //Left Hand  position task
    ChppLosPositionTask handposTL ( *attCentipede,* robotslinks[0],* robots[0]->leftWrist() ,robots[0]->leftHand()->centerInWristFrame(), vector3d ( 0.3,-3.0,1.08 ) );
    handposTL.maxVelocityNorm ( 0.02 );
    handposTL.dampingFactor ( 0.2 );
    tasks.push_back ( &handposTL );
    */

    //Right Hand  position task
    vector3d lpoint;
    robots[0]->rightHand()->getCenter ( lpoint );
    ChppLosPositionTask handposTR ( *attCentipede,* robotslinks[0],* robots[0]->rightWrist() ,lpoint, vector3d ( 0.0,0.0,0.05 ) );
    handposTR.maxVelocityNorm ( 0.02 );
    handposTR.dampingFactor ( 0.1 );
    tasks.push_back ( &handposTR );


    //Right Hand  position task
    ChppLosGazeTask gazeT ( *attCentipede,* robotslinks[0], vector3d ( 0.0,0.0,0.05 ) );
    gazeT.maxVelocityNorm ( 0.05 );
    gazeT.dampingFactor ( 0.1 );
    tasks.push_back ( &gazeT );

    /*
    //Task on waist to remain horizontal
    ChppLosParallelTask parC ( *attCentipede, * robotslinks[0], *robots[0]->waist(),vector3d ( 0,0,1 ),vector3d ( 0,0,1 ) );
    parC.dampingFactor ( 0.2 );
    parC.maxVelocityNorm ( 0.2 );
    tasks.push_back ( &parC );
    */

    //Task on configuration to remain at half-sitting
    ChppLosLinkConfigurationTask configT1 ( *attCentipede,* robotslinks[0], hsc,configMask );
    configT1.dampingFactor ( 0.2 );
    configT1.maxVelocityNorm ( 0.2 );
    ////tasks.push_back ( &configT1 );

    //Centipede constraints (feet sliding on ground, feet anticollision, com in support polygon)
    ChppLosCentipedeFactory::createCentipedeConstraints ( attCentipede, centipedeConstraints );
    std::list<ChppLosTask*>::iterator iter;
    for ( iter = centipedeConstraints.begin();iter != centipedeConstraints.end();iter++ )
        constraints.push_back ( *iter );

    //Create a disc on the ground to avoid
    ChppLosDiscAvoider discAvoider ( *attCentipede );
    discAvoider.setPosition ( 0.0,0.0 );
    discAvoider.setMargins ( 0.04,0.03,0.01 );
    constraints.push_back ( &discAvoider );

    //Simplified collision avoidance constraint for hrp2
    ChppLosCollisionAvoider hrp2AvoiderC ( *attCentipede );
    HRP2Wrapper hrp2 ( *attCentipede,*robotslinks[0] );
    hrp2AvoiderC.setSelfPair ( hrp2.handR(), hrp2.upperlegR(), 0.03,0.0,0.02 );
    hrp2AvoiderC.setSelfPair ( hrp2.lowerarmR(), hrp2.upperlegR(), 0.03,0.0,0.02 );
    hrp2AvoiderC.setSelfPair ( hrp2.handR(), hrp2.lowerlegR(), 0.03,0.0,0.02 );
    hrp2AvoiderC.setSelfPair ( hrp2.lowerarmR(), hrp2.lowerlegR(), 0.03,0.0,0.02 );
    constraints.push_back ( &hrp2AvoiderC );
    
    solver1.constraints ( constraints );
    solver1.tasks ( tasks );
    solver1.setMask ( attMask );

    solver1.solve ();
    dumpSolution("test.motion",solver1.solutionMotion());

    std::list<ChppLosTask*>::iterator iterC = centipedeConstraints.begin();
    while (iterC != centipedeConstraints.end())
    {
        delete *iterC;
        iterC++;
    }

    //return;
    ChppLosMotionGenerator mg ( attpath );

    //attMask = subrange ( standingRobots[0]->maskFactory()->legsMask(),6,46 );
    //mg.setSolverMask(attMask);


    /*
    ChppLosParallelTask mparC ( mg.centipede(), mg.robotLink(), *mg.robot().waist(),vector3d ( 0,0,1 ),vector3d ( 0,0,1 ) );
    mparC.startTime ( 0.0 );
    mparC.endTime ( 14.0 );
    constraints.push_back ( &mparC );
    */

    tasks.clear();
    constraints.clear();

    double timeEnd = 24.0;
    //Simplified collision avoidance constraint for hrp2
    ChppLosCollisionAvoider mhrp2AvoiderC ( mg.centipede() );
    HRP2Wrapper mhrp2 ( mg.centipede(),mg.robotLink() );
    mhrp2AvoiderC.setSelfPair ( mhrp2.handR(), mhrp2.upperlegR(), 0.03,0.0,0.02 );
    mhrp2AvoiderC.setSelfPair ( mhrp2.lowerarmR(), mhrp2.upperlegR(), 0.03,0.0,0.02 );
    mhrp2AvoiderC.setSelfPair ( mhrp2.handR(), mhrp2.lowerlegR(), 0.03,0.0,0.02 );
    mhrp2AvoiderC.setSelfPair ( mhrp2.lowerarmR(), mhrp2.lowerlegR(), 0.03,0.0,0.02 );
    constraints.push_back ( &mhrp2AvoiderC );
    mhrp2AvoiderC.startTime ( 0.0 );
    mhrp2AvoiderC.endTime ( timeEnd );


    ChppLosParallelTask mparC ( mg.centipede(), mg.robotLink(), *mg.robot().waist(),vector3d ( 0,0,1 ),vector3d ( 0,0,1 ) );
    mparC.startTime ( 0.0 );
    mparC.endTime ( 7.0 );
    constraints.push_back ( &mparC );

    //Right Hand  position task
    vector3d lpoint2;
    mg.robot().rightHand()->getCenter ( lpoint2 );
    ChppLosPositionTask mhandposTR ( mg.centipede(),mg.robotLink(),*mg.robot().rightWrist() ,lpoint2, vector3d ( 0.0,0.0,0.05 ) );
    mhandposTR.maxVelocityNorm ( 0.0007 );
    mhandposTR.dampingFactor ( 0.2 );
    mhandposTR.startTime ( 8.0 );
    mhandposTR.endTime ( timeEnd );
    tasks.push_back ( &mhandposTR );


    //Gaze task
    ChppLosGazeTask mgazeT ( mg.centipede(),mg.robotLink(), vector3d ( 0.0,0.0,0.05 ) );
    mgazeT.maxVelocityNorm ( 0.0005 );
    mgazeT.dampingFactor ( 0.2 );
    mgazeT.startTime ( 0.0 );
    mgazeT.endTime ( timeEnd );
    tasks.push_back ( &mgazeT );

    mg.constraints ( constraints );
    mg.tasks ( tasks );

    mg.generate ( *attCentipede,standingRobots[0]->halfsittingConfiguration() );

    //mg.motion().dumpTo ( "bf-wbm" );

    ChppRobotMotion wmotion ( &mg.robot(),0.0,5e-3 );
    wmotion.appendMotion ( mg.motion() );

    ChppGikHandTask handGikTask ( &mg.standingRobot(), 5e-3 );
    handGikTask.forRightHand ( true );
    handGikTask.targetClench ( 0.7 );
    handGikTask.solve();
    wmotion.appendMotion ( handGikTask.solutionMotion() );

    constraints.clear();
    tasks.clear();
    timeEnd = 10.0;
    mparC.startTime ( 0.0 );
    mparC.endTime ( timeEnd );
    mparC.dampingFactor ( 0.1 );
    mparC.maxVelocityNorm ( 0.0005 );
    tasks.push_back ( &mparC );

    ChppLosLinkConfigurationTask mconfigT ( mg.centipede(), mg.robotLink(), hsc, configMask );
    mconfigT.dampingFactor ( 0.2 );
    mconfigT.maxVelocityNorm ( 0.0007 );
    mconfigT.startTime ( 0.0 );
    mconfigT.endTime ( timeEnd );
    tasks.push_back ( &mconfigT );

    mg.constraints ( constraints );
    mg.tasks ( tasks );
    mg.generate ( wmotion.lastSample()->configuration );
    wmotion.appendMotion ( mg.motion() );

    wmotion.dumpTo ( "seq-bf-wbm" );
}

void  ChppLocalStepperTest::dumpSolution(const char* inFilename, const std::list<vectorN>& inSolution)
{
    if (inSolution.empty())
    {
        std::cout << "Nothing to dump" << std::endl;
        return;
    }

    FILE* dumpFileQ = fopen (inFilename, "w");
    
    std::list<vectorN>::const_iterator iter= inSolution.begin();
    
    unsigned int sz = ((vectorN)(*(inSolution.begin()))).size();
    unsigned int i = 0;
    while(iter!=inSolution.end())
    {
        for (i=0;i<sz;i++)
            fprintf (dumpFileQ, "%lf ", (*iter)(i));
        fprintf (dumpFileQ,"\n");
        iter++;
    }
    fclose (dumpFileQ);
}


